import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.awt.image.IndexColorModel;
import java.util.Random;
import java.applet.Applet;

public class Fire2 extends Applet implements Runnable {
    final Random rand = new Random();
    final float[] fireBlur2 = new float[]{
                0.400f, 0.250f, 0.100f,
                0.175f, 0.025f, 0.050f,
                0.000f, 0.000f, 0.000f,
        };
    final float[] fireBlur = new float[]{
                0.175f, 0.175f, 0.130f,
                0.165f, 0.155f, 0.130f,
                0.066f, 0.000f, 0.000f,
        };
    final float[] crosses = new float[] {
                -0.125f,  0.200f, -0.125f,
                 0.200f,  0.750f,  0.200f,
                -0.125f,  0.200f, -0.125f
        };

    BufferedImage flame;
    BufferedImage next;
    BufferedImage display;
    ConvolveOp effectOp;
    Kernel kernel;

    final IndexColorModel indexColorModel;

    public Fire2() {
        byte[] r = new byte[256], g = new byte[256], b = new byte[256];
        for (int i = 0; i < 64; ++i) {
            b[i + 128] = g[i + 64] = r[i] = (byte)(i * 4);
            b[i + 192] = g[i + 128] = r[i+128] = r[i+64] = (byte)255;
            b[i + 64] = b[i] = g[i] = 0;
            r[i + 192] = g[i + 192] = (byte) ((byte) 255 - (r[i + 192] = (byte) (i*4)));
         }
        indexColorModel = new IndexColorModel(8, r.length, r, g, b);
    }

    volatile boolean notDone = true;

    public void resize() {
        synchronized(display) {
            flame = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_BYTE_GRAY);
            next = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_BYTE_GRAY);
            display = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_BYTE_INDEXED, indexColorModel);
        }
    }

    public void init() {
        kernel = new Kernel(3, 3, fireBlur);
        effectOp = new ConvolveOp(kernel);
        flame = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_BYTE_GRAY);
        next = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_BYTE_GRAY);
        display = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_BYTE_INDEXED, indexColorModel);
        new Thread(this).start();
    }

    public void run() {
        while (notDone) {
            step();
            repaint();
            Thread.yield();
        }
    }

    public void stop() {
        notDone = false;
    }


    public void step() {
        synchronized(display) {
            for (int i = 0; i < flame.getWidth()/16; ++i) {
                flame.setRGB(rand.nextInt(flame.getWidth()), flame.getHeight() - 1, 0xFFFFFF);
                flame.setRGB(rand.nextInt(flame.getWidth()), flame.getHeight() - 1, 0);
            }
            effectOp.filter(flame.getRaster(), next.getRaster());
            BufferedImage t = flame;
            flame = next;
            next = t;
            display.setData(flame.getData());
        }
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void paint(Graphics g) {
        synchronized(display) {
            g.drawImage(display, 0, 0, display.getWidth(), display.getHeight()-2,0, 0, display.getWidth(), display.getHeight()-2, null);
        }
    }

    public static void main(String[] args) throws InterruptedException {
        Fire2 effect = new Fire2();
        JFrame frame = new JFrame();
        frame.setSize(640, 480);
        frame.getContentPane().add(effect);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setVisible(true);

    }
}
